from dataclasses import dataclass
from typing import Optional, List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class FinishReason(AI21BaseModelMixin):
    reason: str
    length: Optional[int] = None
    sequence: Optional[str] = None


@dataclass
class ChatOutput(AI21BaseModelMixin):
    text: str
    role: str
    finish_reason: FinishReason


@dataclass
class ChatResponse(AI21BaseModelMixin):
    outputs: List[ChatOutput]



from __future__ import annotations

from abc import ABC
from typing import Any, Dict, Optional, BinaryIO

from ai21.ai21_http_client import AI21HTTPClient


class StudioResource(ABC):
    def __init__(self, client: AI21HTTPClient):
        self._client = client

    def _post(
        self,
        url: str,
        body: Dict[str, Any],
        files: Optional[Dict[str, BinaryIO]] = None,
    ) -> Dict[str, Any]:
        return self._client.execute_http_request(
            method="POST",
            url=url,
            params=body or {},
            files=files,
        )

    def _get(self, url: str, params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        return self._client.execute_http_request(method="GET", url=url, params=params or {})

    def _put(self, url: str, body: Dict[str, Any] = None) -> Dict[str, Any]:
        return self._client.execute_http_request(method="PUT", url=url, params=body or {})

    def _delete(self, url: str) -> Dict[str, Any]:
        return self._client.execute_http_request(
            method="DELETE",
            url=url,
        )



from __future__ import annotations

import json
from abc import ABC
from typing import Any, Dict

from ai21.clients.sagemaker.sagemaker_session import SageMakerSession


class SageMakerResource(ABC):
    def __init__(self, sagemaker_session: SageMakerSession):
        self._sagemaker_session = sagemaker_session

    def _invoke(self, body: Dict[str, Any]) -> Dict[str, Any]:
        return self._sagemaker_session.invoke_endpoint(
            input_json=json.dumps(body),
        )



from typing import List

from ai21.ai21_http_client import AI21HTTPClient
from ai21.clients.sagemaker.constants import (
    SAGEMAKER_MODEL_PACKAGE_NAMES,
)
from ai21.errors import ModelPackageDoesntExistError

_JUMPSTART_ENDPOINT = "jumpstart"
_LIST_VERSIONS_ENDPOINT = f"{_JUMPSTART_ENDPOINT}/list_versions"
_GET_ARN_ENDPOINT = f"{_JUMPSTART_ENDPOINT}/get_model_version_arn"

LATEST_VERSION_STR = "latest"


class SageMaker:
    @classmethod
    def get_model_package_arn(cls, model_name: str, region: str, version: str = LATEST_VERSION_STR) -> str:
        _assert_model_package_exists(model_name=model_name, region=region)

        client = cls._create_ai21_http_client()

        response = client.execute_http_request(
            method="POST",
            url=f"{client.get_base_url()}/{_GET_ARN_ENDPOINT}",
            params={
                "modelName": model_name,
                "region": region,
                "version": version,
            },
        )

        arn = response["arn"]

        if not arn:
            raise ModelPackageDoesntExistError(model_name=model_name, region=region, version=version)

        return arn

    @classmethod
    def list_model_package_versions(cls, model_name: str, region: str) -> List[str]:
        _assert_model_package_exists(model_name=model_name, region=region)

        client = cls._create_ai21_http_client()

        response = client.execute_http_request(
            method="POST",
            url=f"{client.get_base_url()}/{_LIST_VERSIONS_ENDPOINT}",
            params={
                "modelName": model_name,
                "region": region,
            },
        )

        return response["versions"]

    @classmethod
    def _create_ai21_http_client(cls) -> AI21HTTPClient:
        return AI21HTTPClient()


def _assert_model_package_exists(model_name, region):
    if model_name not in SAGEMAKER_MODEL_PACKAGE_NAMES:
        raise ModelPackageDoesntExistError(model_name=model_name, region=region)



from dataclasses import dataclass
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Segment(AI21BaseModelMixin):
    segment_text: str
    segment_type: str


@dataclass
class SegmentationResponse(AI21BaseModelMixin):
    segments: List[Segment]



from typing import Optional, Dict, Any, BinaryIO

from ai21.ai21_env_config import _AI21EnvConfig, AI21EnvConfig
from ai21.errors import MissingApiKeyError
from ai21.http_client import HttpClient
from ai21.version import VERSION


class AI21HTTPClient:
    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        api_host: Optional[str] = None,
        api_version: Optional[str] = None,
        headers: Optional[Dict[str, Any]] = None,
        timeout_sec: Optional[int] = None,
        num_retries: Optional[int] = None,
        organization: Optional[str] = None,
        application: Optional[str] = None,
        via: Optional[str] = None,
        http_client: Optional[HttpClient] = None,
        env_config: _AI21EnvConfig = AI21EnvConfig,
    ):
        self._env_config = env_config
        self._api_key = api_key or self._env_config.api_key

        if self._api_key is None:
            raise MissingApiKeyError()

        self._api_host = api_host or self._env_config.api_host
        self._api_version = api_version or self._env_config.api_version
        self._headers = headers
        self._timeout_sec = timeout_sec or self._env_config.timeout_sec
        self._num_retries = num_retries or self._env_config.num_retries
        self._organization = organization
        self._application = application
        self._via = via

        headers = self._build_headers(passed_headers=headers)
        self._http_client = self._init_http_client(http_client=http_client, headers=headers)

    def _build_headers(self, passed_headers: Optional[Dict[str, Any]]) -> Dict[str, Any]:
        headers = {
            "Content-Type": "application/json",
            "User-Agent": self._build_user_agent(),
        }

        if self._api_key:
            headers["Authorization"] = f"Bearer {self._api_key}"

        if passed_headers is not None:
            headers.update(passed_headers)

        return headers

    def _init_http_client(self, http_client: Optional[HttpClient], headers: Dict[str, Any]) -> HttpClient:
        if http_client is None:
            return HttpClient(
                timeout_sec=self._timeout_sec,
                num_retries=self._num_retries,
                headers=headers,
            )

        http_client.add_headers(headers)

        return http_client

    def _build_user_agent(self) -> str:
        user_agent = f"ai21 studio SDK {VERSION}"

        if self._organization is not None:
            user_agent = f"{user_agent} organization: {self._organization}"

        if self._application is not None:
            user_agent = f"{user_agent} application: {self._application}"

        if self._via is not None:
            user_agent = f"{user_agent} via: {self._via}"

        return user_agent

    def execute_http_request(
        self,
        method: str,
        url: str,
        params: Optional[Dict] = None,
        files: Optional[Dict[str, BinaryIO]] = None,
    ):
        return self._http_client.execute_http_request(method=method, url=url, params=params, files=files)

    def get_base_url(self) -> str:
        return f"{self._api_host}/studio/{self._api_version}"



from abc import ABC, abstractmethod
from typing import List, Any, Dict, Optional

from ai21.resources.responses.embed_response import EmbedResponse


class Embed(ABC):
    _module_name = "embed"

    @abstractmethod
    def create(self, texts: List[str], *, type: Optional[str] = None, **kwargs) -> EmbedResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> EmbedResponse:
        return EmbedResponse.from_dict(json)

    def _create_body(self, texts: List[str], type: Optional[str]) -> Dict[str, Any]:
        return {"texts": texts, "type": type}



from __future__ import annotations

from typing import Optional

from ai21.resources.responses.summarize_response import SummarizeResponse
from ai21.resources.sagemaker_resource import SageMakerResource
from ai21.resources.bases.summarize_base import Summarize


class SageMakerSummarize(SageMakerResource, Summarize):
    def create(
        self,
        source: str,
        source_type: str,
        *,
        focus: Optional[str] = None,
        summary_method: Optional[str] = None,
        **kwargs,
    ) -> SummarizeResponse:
        body = self._create_body(
            source=source,
            source_type=source_type,
            focus=focus,
            summary_method=summary_method,
        )

        response = self._invoke(body)

        return self._json_to_response(response)



from typing import Optional, Dict, Any, List

from ai21.resources.bases.completion_base import Completion
from ai21.resources.responses.completion_response import CompletionsResponse
from ai21.resources.studio_resource import StudioResource


class StudioCompletion(StudioResource, Completion):
    def create(
        self,
        model: str,
        prompt: str,
        *,
        max_tokens: Optional[int] = None,
        num_results: Optional[int] = 1,
        min_tokens: Optional[int] = 0,
        temperature: Optional[float] = 0.7,
        top_p: Optional[float] = 1,
        top_k_return: Optional[int] = 0,
        custom_model: Optional[str] = None,
        stop_sequences: Optional[List[str]] = None,
        frequency_penalty: Optional[Dict[str, Any]] = None,
        presence_penalty: Optional[Dict[str, Any]] = None,
        count_penalty: Optional[Dict[str, Any]] = None,
        epoch: Optional[int] = None,
        **kwargs,
    ) -> CompletionsResponse:
        url = f"{self._client.get_base_url()}/{model}"

        if custom_model is not None:
            url = f"{url}/{custom_model}"

        url = f"{url}/{self._module_name}"
        body = self._create_body(
            model=model,
            prompt=prompt,
            max_tokens=max_tokens,
            num_results=num_results,
            min_tokens=min_tokens,
            temperature=temperature,
            top_p=top_p,
            top_k_return=top_k_return,
            custom_model=custom_model,
            stop_sequences=stop_sequences,
            frequency_penalty=frequency_penalty,
            presence_penalty=presence_penalty,
            count_penalty=count_penalty,
            epoch=epoch,
        )
        return self._json_to_response(self._post(url=url, body=body))



from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from ai21.resources.responses.dataset_response import DatasetResponse


class Dataset(ABC):
    _module_name = "dataset"

    @abstractmethod
    def create(
        self,
        file_path: str,
        dataset_name: str,
        *,
        selected_columns: Optional[str] = None,
        approve_whitespace_correction: Optional[bool] = None,
        delete_long_rows: Optional[bool] = None,
        split_ratio: Optional[float] = None,
        **kwargs,
    ):
        pass

    @abstractmethod
    def list(self):
        pass

    @abstractmethod
    def get(self, dataset_pid: str):
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> DatasetResponse:
        return DatasetResponse.from_dict(json)

    def _create_body(
        self,
        dataset_name: str,
        selected_columns: Optional[str],
        approve_whitespace_correction: Optional[bool],
        delete_long_rows: Optional[bool],
        split_ratio: Optional[float],
    ) -> Dict[str, Any]:
        return {
            "dataset_name": dataset_name,
            "selected_columns": selected_columns,
            "approve_whitespace_correction": approve_whitespace_correction,
            "delete_long_rows": delete_long_rows,
            "split_ratio": split_ratio,
        }



from typing import Any

from ai21.clients.studio.ai21_client import AI21Client
from ai21.errors import (
    AI21APIError,
    APITimeoutError,
    MissingApiKeyError,
    ModelPackageDoesntExistError,
    AI21Error,
    TooManyRequestsError,
)
from ai21.logger import setup_logger
from ai21.resources.responses.answer_response import AnswerResponse
from ai21.resources.responses.chat_response import ChatResponse
from ai21.resources.responses.completion_response import CompletionsResponse
from ai21.resources.responses.custom_model_response import CustomBaseModelResponse
from ai21.resources.responses.dataset_response import DatasetResponse
from ai21.resources.responses.embed_response import EmbedResponse
from ai21.resources.responses.file_response import FileResponse
from ai21.resources.responses.gec_response import GECResponse
from ai21.resources.responses.improvement_response import ImprovementsResponse
from ai21.resources.responses.library_answer_response import LibraryAnswerResponse
from ai21.resources.responses.library_search_response import LibrarySearchResponse
from ai21.resources.responses.paraphrase_response import ParaphraseResponse
from ai21.resources.responses.segmentation_response import SegmentationResponse
from ai21.resources.responses.summarize_by_segment_response import SummarizeBySegmentResponse
from ai21.resources.responses.summarize_response import SummarizeResponse
from ai21.services.sagemaker import SageMaker
from ai21.version import VERSION

__version__ = VERSION
setup_logger()


def _import_bedrock_client():
    from ai21.clients.bedrock.ai21_bedrock_client import AI21BedrockClient

    return AI21BedrockClient


def _import_sagemaker_client():
    from ai21.clients.sagemaker.ai21_sagemaker_client import AI21SageMakerClient

    return AI21SageMakerClient


def _import_bedrock_model_id():
    from ai21.clients.bedrock.bedrock_model_id import BedrockModelID

    return BedrockModelID


def __getattr__(name: str) -> Any:
    try:
        if name == "AI21BedrockClient":
            return _import_bedrock_client()

        if name == "AI21SageMakerClient":
            return _import_sagemaker_client()

        if name == "BedrockModelID":
            return _import_bedrock_model_id()
    except ImportError as e:
        raise ImportError(f'Please install "ai21[AWS]" in order to use {name}') from e


__all__ = [
    "AI21Client",
    "AI21APIError",
    "APITimeoutError",
    "AI21Error",
    "MissingApiKeyError",
    "ModelPackageDoesntExistError",
    "TooManyRequestsError",
    "AI21BedrockClient",
    "AI21SageMakerClient",
    "BedrockModelID",
    "AnswerResponse",
    "ChatResponse",
    "CompletionsResponse",
    "CustomBaseModelResponse",
    "DatasetResponse",
    "EmbedResponse",
    "FileResponse",
    "GECResponse",
    "ImprovementsResponse",
    "LibraryAnswerResponse",
    "LibrarySearchResponse",
    "ParaphraseResponse",
    "SageMaker",
    "SegmentationResponse",
    "SummarizeBySegmentResponse",
    "SummarizeResponse",
]



from typing import Optional

import boto3

from ai21.ai21_env_config import _AI21EnvConfig, AI21EnvConfig
from ai21.clients.sagemaker.resources.sagemaker_answer import SageMakerAnswer
from ai21.clients.sagemaker.resources.sagemaker_completion import SageMakerCompletion
from ai21.clients.sagemaker.resources.sagemaker_gec import SageMakerGEC
from ai21.clients.sagemaker.resources.sagemaker_paraphrase import SageMakerParaphrase
from ai21.clients.sagemaker.resources.sagemaker_summarize import SageMakerSummarize
from ai21.clients.sagemaker.sagemaker_session import SageMakerSession


class AI21SageMakerClient:
    """
    :param endpoint_name: The name of the endpoint to use for the client.
    :param region: The AWS region of the endpoint.
    :param session: An optional boto3 session to use for the client.
    """

    def __init__(
        self,
        endpoint_name: str,
        region: Optional[str] = None,
        session: Optional["boto3.Session"] = None,
        env_config: _AI21EnvConfig = AI21EnvConfig,
        **kwargs,
    ):

        self._env_config = env_config
        _session = ()
        self._session = SageMakerSession(
            session=session, region=region or self._env_config.aws_region, endpoint_name=endpoint_name
        )
        self.completion = SageMakerCompletion(self._session)
        self.paraphrase = SageMakerParaphrase(self._session)
        self.answer = SageMakerAnswer(self._session)
        self.gec = SageMakerGEC(self._session)
        self.summarize = SageMakerSummarize(self._session)



from abc import ABC, abstractmethod
from typing import Optional, List, Any, Dict

from ai21.resources.responses.custom_model_response import CustomBaseModelResponse


class CustomModel(ABC):
    _module_name = "custom-model"

    @abstractmethod
    def create(
        self,
        dataset_id: str,
        model_name: str,
        model_type: str,
        *,
        learning_rate: Optional[float] = None,
        num_epochs: Optional[int] = None,
        **kwargs,
    ) -> None:
        pass

    @abstractmethod
    def list(self) -> List[CustomBaseModelResponse]:
        pass

    @abstractmethod
    def get(self, resource_id: str) -> CustomBaseModelResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> CustomBaseModelResponse:
        return CustomBaseModelResponse.from_dict(json)

    def _create_body(
        self,
        dataset_id: str,
        model_name: str,
        model_type: str,
        learning_rate: Optional[float],
        num_epochs: Optional[int],
    ) -> Dict[str, Any]:
        return {
            "dataset_id": dataset_id,
            "model_name": model_name,
            "model_type": model_type,
            "learning_rate": learning_rate,
            "num_epochs": num_epochs,
        }



from typing import Optional

from ai21.resources.bases.summarize_base import Summarize
from ai21.resources.responses.summarize_response import SummarizeResponse
from ai21.resources.studio_resource import StudioResource


class StudioSummarize(StudioResource, Summarize):
    _module_name = "summarize"

    def create(
        self,
        source: str,
        source_type: str,
        *,
        focus: Optional[str] = None,
        summary_method: Optional[str] = None,
        **kwargs,
    ) -> SummarizeResponse:
        # Make a summarize request to the AI21 API. Returns the response either as a string or a AI21Summarize object.
        body = self._create_body(
            source=source,
            source_type=source_type,
            focus=focus,
            summary_method=summary_method,
        )
        url = f"{self._client.get_base_url()}/{self._module_name}"
        response = self._post(url=url, body=body)

        return self._json_to_response(response)



from dataclasses import dataclass
from datetime import date
from typing import Optional, List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class FileResponse(AI21BaseModelMixin):
    file_id: str
    name: str
    file_type: str
    size_bytes: int
    created_by: str
    creation_date: date
    last_updated: date
    status: str
    path: Optional[str] = None
    labels: Optional[List[str]] = None
    public_url: Optional[str] = None
    error_code: Optional[int] = None
    error_message: Optional[str] = None



from abc import ABC, abstractmethod
from typing import Any, Dict

from ai21.resources.responses.segmentation_response import SegmentationResponse


class Segmentation(ABC):
    _module_name = "segmentation"

    @abstractmethod
    def create(self, source: str, source_type: str):
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> SegmentationResponse:
        return SegmentationResponse.from_dict(json)

    def _create_body(self, source: str, source_type: str) -> Dict[str, Any]:
        return {"source": source, "sourceType": source_type}



from typing import List

from ai21_tokenizer import BaseTokenizer


class AI21Tokenizer:
    """
    A class that wraps a tokenizer and provides additional functionality.
    """

    def __init__(self, tokenizer: BaseTokenizer):
        self._tokenizer = tokenizer

    def count_tokens(self, text: str) -> int:
        encoded_text = self._tokenizer.encode(text)

        return len(encoded_text)

    def tokenize(self, text: str) -> List[str]:
        encoded_text = self._tokenizer.encode(text)

        return self._tokenizer.convert_ids_to_tokens(encoded_text)



from abc import ABC
from typing import Optional, Any, Dict

from ai21.resources.responses.answer_response import AnswerResponse


class Answer(ABC):
    _module_name = "answer"

    def create(
        self,
        context: str,
        question: str,
        *,
        answer_length: Optional[str] = None,
        mode: Optional[str] = None,
        **kwargs,
    ) -> AnswerResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> AnswerResponse:
        return AnswerResponse.from_dict(json)

    def _create_body(
        self,
        context: str,
        question: str,
        answer_length: Optional[str],
        mode: Optional[str],
    ) -> Dict[str, Any]:
        return {"context": context, "question": question, "answerLength": answer_length, "mode": mode}



from typing import Optional

from ai21.resources.bases.answer_base import Answer
from ai21.resources.responses.answer_response import AnswerResponse
from ai21.resources.sagemaker_resource import SageMakerResource


class SageMakerAnswer(SageMakerResource, Answer):
    def create(
        self,
        context: str,
        question: str,
        *,
        answer_length: Optional[str] = None,
        mode: Optional[str] = None,
        **kwargs,
    ) -> AnswerResponse:
        body = self._create_body(context=context, question=question, answer_length=answer_length, mode=mode)
        response = self._invoke(body)

        return self._json_to_response(response)



import json
from typing import Optional, Dict, Any, BinaryIO

import requests
from requests.adapters import HTTPAdapter, Retry, RetryError

from ai21.errors import (
    BadRequest,
    Unauthorized,
    UnprocessableEntity,
    TooManyRequestsError,
    AI21ServerError,
    ServiceUnavailable,
    AI21APIError,
)
from ai21.logger import logger

DEFAULT_TIMEOUT_SEC = 300
DEFAULT_NUM_RETRIES = 0
RETRY_BACK_OFF_FACTOR = 0.5
TIME_BETWEEN_RETRIES = 1000
RETRY_ERROR_CODES = (429, 500, 503)
RETRY_METHOD_WHITELIST = ["GET", "POST", "PUT"]


def handle_non_success_response(status_code: int, response_text: str):
    if status_code == 400:
        raise BadRequest(details=response_text)
    if status_code == 401:
        raise Unauthorized(details=response_text)
    if status_code == 422:
        raise UnprocessableEntity(details=response_text)
    if status_code == 429:
        raise TooManyRequestsError(details=response_text)
    if status_code == 500:
        raise AI21ServerError(details=response_text)
    if status_code == 503:
        raise ServiceUnavailable(details=response_text)
    raise AI21APIError(status_code, details=response_text)


def requests_retry_session(session, retries=0):
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=RETRY_BACK_OFF_FACTOR,
        status_forcelist=RETRY_ERROR_CODES,
        method_whitelist=frozenset(RETRY_METHOD_WHITELIST),
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount("https://", adapter)
    session.mount("http://", adapter)
    return session


class HttpClient:
    def __init__(
        self,
        session: Optional[requests.Session] = None,
        timeout_sec: int = None,
        num_retries: int = None,
        headers: Dict = None,
    ):
        self._timeout_sec = timeout_sec or DEFAULT_TIMEOUT_SEC
        self._num_retries = num_retries or DEFAULT_NUM_RETRIES
        self._headers = headers or {}
        self._apply_retry_policy = self._num_retries > 0
        self._session = self._init_session(session)

    def execute_http_request(
        self,
        method: str,
        url: str,
        params: Optional[Dict] = None,
        files: Optional[Dict[str, BinaryIO]] = None,
    ):
        timeout = self._timeout_sec
        headers = self._headers
        data = json.dumps(params).encode()
        logger.info(f"Calling {method} {url} {headers} {data}")
        try:
            if method == "GET":
                response = self._session.request(
                    method=method,
                    url=url,
                    headers=headers,
                    timeout=timeout,
                    params=params,
                )
            elif files is not None:
                if method != "POST":
                    raise ValueError(
                        f"execute_http_request supports only POST for files upload, but {method} was supplied instead"
                    )
                if "Content-Type" in headers:
                    headers.pop(
                        "Content-Type"
                    )  # multipart/form-data 'Content-Type' is being added when passing rb files and payload
                response = self._session.request(
                    method=method,
                    url=url,
                    headers=headers,
                    data=params,
                    files=files,
                    timeout=timeout,
                )
            else:
                response = self._session.request(method=method, url=url, headers=headers, data=data, timeout=timeout)
        except ConnectionError as connection_error:
            logger.error(f"Calling {method} {url} failed with ConnectionError: {connection_error}")
            raise connection_error
        except RetryError as retry_error:
            logger.error(
                f"Calling {method} {url} failed with RetryError after {self._num_retries} attempts: {retry_error}"
            )
            raise retry_error
        except Exception as exception:
            logger.error(f"Calling {method} {url} failed with Exception: {exception}")
            raise exception

        if response.status_code != 200:
            logger.error(f"Calling {method} {url} failed with a non-200 response code: {response.status_code}")
            handle_non_success_response(response.status_code, response.text)

        return response.json()

    def _init_session(self, session: Optional[requests.Session]) -> requests.Session:
        if session is not None:
            return session

        return (
            requests_retry_session(requests.Session(), retries=self._num_retries)
            if self._apply_retry_policy
            else requests.Session()
        )

    def add_headers(self, headers: Dict[str, Any]) -> None:
        self._headers.update(headers)



from ai21.resources.bases.gec_base import GEC
from ai21.resources.responses.gec_response import GECResponse
from ai21.resources.studio_resource import StudioResource


class StudioGEC(StudioResource, GEC):
    def create(self, text: str, **kwargs) -> GECResponse:
        body = self._create_body(text=text)
        url = f"{self._client.get_base_url()}/{self._module_name}"
        response = self._post(url=url, body=body)

        return self._json_to_response(response)



from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from ai21.resources.responses.paraphrase_response import ParaphraseResponse


class Paraphrase(ABC):
    _module_name = "paraphrase"

    @abstractmethod
    def create(
        self,
        text: str,
        *,
        style: Optional[str] = None,
        start_index: Optional[int] = 0,
        end_index: Optional[int] = None,
        **kwargs,
    ) -> ParaphraseResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> ParaphraseResponse:
        return ParaphraseResponse.from_dict(json)

    def _create_body(
        self,
        text: str,
        style: Optional[str],
        start_index: Optional[int],
        end_index: Optional[int],
    ) -> Dict[str, Any]:
        return {
            "text": text,
            "style": style,
            "startIndex": start_index,
            "endIndex": end_index,
        }



from dataclasses import dataclass
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Improvement(AI21BaseModelMixin):
    suggestions: List[str]
    start_index: int
    end_index: int
    original_text: str
    improvement_type: str


@dataclass
class ImprovementsResponse(AI21BaseModelMixin):
    id: str
    improvements: List[Improvement]



from abc import ABC, abstractmethod
from typing import Any, Dict, List

from ai21.resources.responses.improvement_response import ImprovementsResponse


class Improvements(ABC):
    _module_name = "improvements"

    @abstractmethod
    def create(self, text: str, types: List[str], **kwargs) -> ImprovementsResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> ImprovementsResponse:
        return ImprovementsResponse.from_dict(json)

    def _create_body(self, text: str, types: List[str]) -> Dict[str, Any]:
        return {"text": text, "types": types}



from dataclasses import dataclass
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Correction(AI21BaseModelMixin):
    suggestion: str
    start_index: int
    end_index: int
    original_text: str
    correction_type: str


@dataclass
class GECResponse(AI21BaseModelMixin):
    id: str
    corrections: List[Correction]



from dataclasses import dataclass
from datetime import datetime

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class DatasetResponse(AI21BaseModelMixin):
    id: str
    dataset_name: str
    size_bytes: int
    creation_date: datetime
    num_examples: int
    validation_num_examples: int
    train_num_examples: int
    num_models_used: int



from typing import List, Optional

from ai21.resources.bases.custom_model_base import CustomModel
from ai21.resources.responses.custom_model_response import CustomBaseModelResponse
from ai21.resources.studio_resource import StudioResource


class StudioCustomModel(StudioResource, CustomModel):
    def create(
        self,
        dataset_id: str,
        model_name: str,
        model_type: str,
        *,
        learning_rate: Optional[float] = None,
        num_epochs: Optional[int] = None,
        **kwargs,
    ) -> None:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        body = self._create_body(
            dataset_id=dataset_id,
            model_name=model_name,
            model_type=model_type,
            learning_rate=learning_rate,
            num_epochs=num_epochs,
        )
        self._post(url=url, body=body)

    def list(self) -> List[CustomBaseModelResponse]:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        response = self._get(url=url)

        return [self._json_to_response(r) for r in response]

    def get(self, resource_id: str) -> CustomBaseModelResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}/{resource_id}"
        return self._json_to_response(self._get(url=url))



from typing import Optional

from ai21.resources.responses.summarize_by_segment_response import (
    SummarizeBySegmentResponse,
)
from ai21.resources.studio_resource import StudioResource
from ai21.resources.bases.summarize_by_segment_base import SummarizeBySegment


class StudioSummarizeBySegment(StudioResource, SummarizeBySegment):
    def create(
        self, source: str, source_type: str, *, focus: Optional[str] = None, **kwargs
    ) -> SummarizeBySegmentResponse:
        body = self._create_body(
            source=source,
            source_type=source_type,
            focus=focus,
        )
        url = f"{self._client.get_base_url()}/{self._module_name}"
        response = self._post(url=url, body=body)
        return self._json_to_response(response)



from abc import ABC, abstractmethod
from typing import Optional, List, Dict, Any

from ai21.resources.responses.completion_response import CompletionsResponse


class Completion(ABC):
    _module_name = "complete"

    @abstractmethod
    def create(
        self,
        model: str,
        prompt: str,
        *,
        max_tokens: int = 64,
        num_results: int = 1,
        min_tokens=0,
        temperature=0.7,
        top_p=1,
        top_k_return=0,
        custom_model: Optional[str] = None,
        stop_sequences: Optional[List[str]] = (),
        frequency_penalty: Optional[Dict[str, Any]] = {},
        presence_penalty: Optional[Dict[str, Any]] = {},
        count_penalty: Optional[Dict[str, Any]] = {},
        epoch: Optional[int] = None,
        **kwargs,
    ) -> CompletionsResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> CompletionsResponse:
        return CompletionsResponse.from_dict(json)

    def _create_body(
        self,
        model: str,
        prompt: str,
        max_tokens: Optional[int],
        num_results: Optional[int],
        min_tokens: Optional[int],
        temperature: Optional[float],
        top_p: Optional[int],
        top_k_return: Optional[int],
        custom_model: Optional[str],
        stop_sequences: Optional[List[str]],
        frequency_penalty: Optional[Dict[str, Any]],
        presence_penalty: Optional[Dict[str, Any]],
        count_penalty: Optional[Dict[str, Any]],
        epoch: Optional[int],
    ):
        return {
            "model": model,
            "customModel": custom_model,
            "prompt": prompt,
            "maxTokens": max_tokens,
            "numResults": num_results,
            "minTokens": min_tokens,
            "temperature": temperature,
            "topP": top_p,
            "topKReturn": top_k_return,
            "stopSequences": stop_sequences or [],
            "frequencyPenalty": frequency_penalty,
            "presencePenalty": presence_penalty,
            "countPenalty": count_penalty,
            "epoch": epoch,
        }
